# -*- coding: utf-8 -*-

# ------------------ 常规通用配置参数 ------------------#

# 并发数
PREFETCH_COUNT = 10

# 最大优先级数
X_MAX_PRIORITY = 15

# 超时时间设置
TIME_OUT = 15

# 最大重试次数
MAX_REQUEST = 4

# 允许重试的状态码
RETRY_HTTP_CODE = [
    209, 301, 302, 400, 403, 404, 405, 408, 412, 429, 500, 502, 503, 504, 505, 521
]

# 是否开启UA池代理
UA_PROXY = True

# 是否开启ip代理
IS_PROXY = False

# 是否开启同一ip会话
IS_SAMEIP = False

# 代理白名单，不器用ip代理时配置不生效
PROXY_WHITELIST = ['127.0.0.1']

# ------------------ mysql连接配置参数 ------------------#

# 主mysql连接配置项
MYSQL_CONFIG = {
    'MYSQL_HOST': 'host',
    'MYSQL_DBNAME': 'database',
    'MYSQL_USER': 'username',
    'MYSQL_PASSWORD': 'password',
    'PORT': 'port'  # 数字类型
}

# 副mysql连接配置项
MYSQL_BAKCUP = {
    'MYSQL_HOST': 'host',
    'MYSQL_DBNAME': 'database',
    'MYSQL_USER': 'username',
    'MYSQL_PASSWORD': 'password',
    'PORT': 'port'  # 数字类型
}

# 是否开启mysql连接
MYSQL_ENABLED = False

# 是否开启第二个数据库连接
MYSQL_BAKCUP_ENABLED = False

# ------------------ mongodb连接配置参数（待更新） ------------------#

# # mongodb连接地址
# MONGO_CONFIG = {
#     'MONGODB_HOST': "your_host",
#     'MONGODB_PORT': "your_port",
#     'MONGODB_BASE': "your_base"
# }
#
# # 是否连接mongodb数据库
# MONGO_ENABLED = False

# ------------------ redis连接配置参数 ------------------#

# redis地址
REDIS_HOST_LISTS = [{'host': 'port'}]  # port注意是数字类型

# redis用户名和密码，没有密码不用设置
REDIS_ACCOUNT = {
    'username': 'username',
    'password': 'password'
}

# 是否开启redis连接
REDIS_ENABLED = False

# 是否启用去重模式（仅redis队列可用）
FILTER = False

# ------------------ RabbitMQ连接配置参数 ------------------#

# rabbitmq地址及用户名密码
RABBITMQ_CONFIG = {
    'username': 'username',
    'password': 'password',
    'host': 'host',
    'port': 'port'  # 数字类型
}

# 消息淘汰时间，也被称为缓存淘汰机制，超过设定的时间队列中的消息将自动删除（单位秒）
X_MESSAGE_TTL = 86400000

# 是否开启Rabbitmq连接
MQ_ENABLED = False

# ------------------ 进程启动和终止逻辑配置参数 ------------------#

# 重启是否自动清空队列,开启后将不会有断点功能
QUEUE_AUTO_CLEAR = True

# 是否开启异步生产，默认为一边生产一边消费
ASYNC_PROD = True

# 空转时间，允许队列最大空置时间(秒),超时队列为空进程将结束，切记要比请求超时时间长
WAITTING_TIME = 50

# 自动关闭程序最大延迟时间
DELAY_TIME = 4

# 指定线上服务器系统类型，用于区分队列名称，防止本地和线上共享队列，不区分大小写
ONLINE_SYS = 'linux'

# ------------------ 日志配置参数 ------------------#

# 日志保存路径
LOG_PATH = ''

# 日志级别
LOG_LEVEL = 'DEBUG'

# ------------------ 中间件配置参数 ------------------#

# 自定义的中间件，可创建多个，数字越小优先级越高
MIDDLEWARES = {
    # 'UaMiddleware': 200,
    # 'TimerMiddleware': 100
}

# ------------------ 数据管道配置参数 ------------------#

# 数据处理管道，可创建多个，数字越小优先级越高
ITEM_PIPELINES = {
    # 'MyProject1Pipeline': 100
}
